/*=====================================================================================================================

   " ".

    .

=======================================================================================================================

  :

  26.02.2015 [*]    (  ).
  25.02.2015 [+]    ( ).
             [*] ViewDocumentPowerName:   .
  15.02.2015 [*] GridDocumentList:     
  09.04.2012 [+] ViewDocumentPowerName:     .
  24.03.2012 [-] DeleteDocument:   .
  07.08.2009 [+] DeleteDocument:  ,    .
  26.07.2007 [+] GridDocumentPosition:  .
  26.03.2007 [+] GridDocumentList:  .
             [+]  .
  09.10.1997 [!]  .

=======================================================================================================================
  Copyright 19972012 TSK Developer Laboratory, Moscow.                                         Konstantin Tsvetkov
                                                                                                  e-mail: dev@tsklab.ru
                                                                                                  http://www.tsklab.ru
  CVS: $Id: scDocument.sql,v 1.8 2015/02/26 15:34:44 tsk Exp $
=====================================================================================================================*/
USE ComputerWare
--
EXECUTE CheckDropProcedure  'GridDocumentList'
EXECUTE CheckDropProcedure  'GridDocumentPosition'
EXECUTE CheckDropView		    'ViewDocumentPowerName'
EXECUTE CheckDropTrigger    'DocumentInsert'
EXECUTE CheckDropTrigger    'DocumentUpdate'
EXECUTE CheckDropTrigger    'DocumentDelete'
--
GO
--=====================================================================================================================
CREATE PROCEDURE GridDocumentList
AS
  SELECT Document.IdDocument AS ID, 
         Document.DocDate, 
         ISNULL(DocKind.Name + ' ', '') + ' ' + ISNULL(Document.DocNumber, '[/]') AS DocName, 
         FirmNamePower.NameTrademark AS ClientName, 
         (SELECT SUM(CompleteSet.Price) FROM CompleteSet 
            WHERE Document.IdDocument = CompleteSet.DocumentId) AS [Sum]
    FROM Document 
      LEFT OUTER JOIN FirmNamePower ON Document.DocClient = FirmNamePower.ID
      LEFT OUTER JOIN DocKind ON Document.DocKind = DocKind.ID
    ORDER BY Document.DocDate DESC, Document.DocNumber DESC
--
GO
--=====================================================================================================================
CREATE PROCEDURE GridDocumentPosition @DOC INT
AS
  SELECT CompleteSet.IdComponent AS ID, 
         CompleteSet.DocumentPosition AS Position, 
         Kind.ShortName AS Kind, 
         Model.DisplayName AS Model, 
         CompleteSet.DisplayNumber AS Number, 
         CompleteSet.Price, 
         LookupDevice.Department, LookupDevice.Place, LookupDevice.Device, 
         LookupCompleteSet.Last, LookupCompleteSet.Warranty, LookupCompleteSet.Manufact,
         CompleteSet.CmplIdDevice as IDDevice
    FROM CompleteSet 
      INNER JOIN Model ON CompleteSet.CmplIdModel = Model.IdModel 
      INNER JOIN Kind ON Model.ModlKind = Kind.IdKind 
      INNER JOIN LookupDevice ON CompleteSet.CmplIdDevice = LookupDevice.ID 
      INNER JOIN LookupCompleteSet ON CompleteSet.IdComponent = LookupCompleteSet.ID
    WHERE (CompleteSet.DocumentId = @DOC)
    ORDER BY CompleteSet.DocumentPosition, CompleteSet.DisplayNumber
--
GO
--=====================================================================================================================
CREATE VIEW ViewDocumentPowerName
AS
  SELECT [Document].IdDocument AS ID, 
         ISNULL(DocKind.Name + ' ', ' ') + ISNULL(' ' + [Document].DocNumber + ' ', '') 
           + ISNULL(' ' + CONVERT(VARCHAR, [Document].DocDate, 104), '') AS NameNumber, 
         ISNULL(DocKind.Name + ' ', ' ') + ISNULL(' ' + [Document].DocNumber + ' ', '') 
           + ISNULL(' ' + CONVERT(VARCHAR, [Document].DocDate, 104), '') 
           + ISNULL('; ' + Trademark.Name + '', '') 
           + ISNULL(', ' + Client.ClntNameFull, '') AS NameFirm
    FROM [Document] 
      INNER JOIN DocKind ON [Document].DocKind = DocKind.ID 
      LEFT OUTER JOIN Client ON [Document].DocClient = Client.IdClient
      LEFT OUTER JOIN Trademark ON Client.Trademark = Trademark.ID
--
GO
--=====================================================================================================================
CREATE TRIGGER DocumentInsert ON Document FOR INSERT
AS
  UPDATE DocKind SET Uses = (SELECT COUNT(*) FROM Document WHERE (Document.DocKind = inserted.DocKind))
    FROM DocKind INNER JOIN inserted ON DocKind.ID = inserted.DocKind
  --
  UPDATE Client SET CountDoc = (SELECT COUNT(*) FROM Document WHERE (Document.DocClient = inserted.DocClient))
    FROM Client INNER JOIN inserted ON Client.IdClient = inserted.DocClient
--
GO
--=====================================================================================================================
CREATE TRIGGER DocumentUpdate ON Document FOR UPDATE
AS
  UPDATE DocKind SET Uses = (SELECT COUNT(*) FROM Document WHERE (Document.DocKind = inserted.DocKind))
    FROM DocKind INNER JOIN inserted ON DocKind.ID = inserted.DocKind
  UPDATE DocKind SET Uses = (SELECT COUNT(*) FROM Document WHERE (Document.DocKind = deleted.DocKind))
    FROM DocKind INNER JOIN deleted ON DocKind.ID = deleted.DocKind
  --
  UPDATE Client SET CountDoc = (SELECT COUNT(*) FROM Document WHERE (Document.DocClient = inserted.DocClient))
  FROM Client INNER JOIN inserted ON Client.IdClient = inserted.DocClient
  UPDATE Client SET CountDoc = (SELECT COUNT(*) FROM Document WHERE (Document.DocClient = deleted.DocClient))
  FROM Client INNER JOIN deleted ON Client.IdClient = deleted.DocClient

--
GO
--=====================================================================================================================
CREATE TRIGGER DocumentDelete ON Document FOR DELETE
AS
  UPDATE DocKind SET Uses = (SELECT COUNT(*) FROM Document WHERE (Document.DocKind = deleted.DocKind))
    FROM DocKind INNER JOIN deleted ON DocKind.ID = deleted.DocKind
  --
  UPDATE Client SET CountDoc = (SELECT COUNT(*) FROM Document WHERE (Document.DocClient = deleted.DocClient))
  FROM Client INNER JOIN deleted ON Client.IdClient = deleted.DocClient
--
GO
--=====================================================================================================================
-- 
--EXECUTE GridDocumentList
--EXECUTE GridDocumentPosition 573
SELECT * FROM ViewDocumentPowerName